<?php

namespace Tygh\Addons\SwBifitOnline;

use Tygh\Registry;
use Tygh\Addons\SwBifitOnline\Logs;
use Tygh\Http;
use Exception;

/**
 * Bifit Online
 */
class Client
{
    /** @var string */
    protected $api_url;
    
    /** @var string */
    protected $test_api_url;

    /** @var string  */
    protected $key_storage;

    /** @var string */
    protected $expires_in;

    /** @var test_token */
    protected $test_token;

    /** @var string */
    protected $token;

    /** @var string */
    protected $mode;

    /** @var string */
    protected $api_version;

    /** @var string */
    protected $company_email;
    
    /** @var array */
    protected $addon_settings;

    /**
     * SwBifitOnline constructor
     */
    public function __construct($mode = 'test', $api_version = '2', $company_email = 'admin@example.com'
    ) {
        
        $this->api_url = 'https://fp.bifit.com/processing-api';
        $this->test_api_url = 'https://fp-test.bifit.com/processing-api';
        
        $this->addon_settings = Registry::get('addons.sw_bifit_online');
        
        $this->mode = $this->addon_settings['mode'];
        
        $this->api_version = $api_version;
        $this->company_email = $company_email;
        $this->key_storage = 'bifit_processing';
        $this->expires_in = 3599 + 3600;
        
        $this->token = $this->addon_settings['token'];
        $this->test_token = 'P5cKbUUD9uSSrSlGdzspLblvBnD0GzTAE0cLmAPSEMxJ79DtLE';
        
        
        $this->Logs = new Logs();
    }

    /**
     * Gets auth token.
     *
     * @return null|string
     */
    public function getToken()
    {
        $this->token = $this->auth();
        
        return $this->token;
    }
    /**
     * Get API URL
     *
     * @return string
     */
    private function getApiUrl()
    {
        if ($this->mode == 'test') {
            $url = $this->test_api_url;
        } else {
             $url = $this->api_url;
        }
        return $url;
    }

    /**
     * Authorize user.
     *
     * @return TokenResponse
     */
    public function auth()
    {
        $url = $this->getApiUrl() . '/oauth/token';
         
        if ($this->mode == 'test') {
            $data = array(
                'token' => $this->test_token,
                'client_id' => 'processing-connector-token',
                'client_secret' => 'processing-connector-token',
                'grant_type' => 'token'
            );
        }else{
            $data = array(
                'token' => $this->token,
                'client_id' => 'processing-connector-token',
                'client_secret' => 'processing-connector-token',
                'grant_type' => 'token'
            );
        }

        $is_expired = true;
        $expired_time = fn_get_storage_data($this->key_storage);
        
        if($expired_time < TIME){
            $is_expired = false;
        }
        
        $response = $this->call($url, $data,  false, false, false, 'authToken');

        if(isset($response['access_token']) && $is_expired == false){
            
            $storage_data = fn_get_storage_data($this->key_storage);
            $response['timestamp'] = TIME;
            
            if(empty($storage_data)){
                fn_set_storage_data($this->key_storage, TIME + $response['expires_in']);
            }
            
            if($is_expired == false){
                $data_ref = array(
                    'refresh_token' => $response['refresh_token'],
                    'client_id'     => $data['client_id'],
                    'client_secret' => $data['client_secret'],
                );
                
                $response = $this->refreshToken($data_ref);
                $response['timestamp'] = TIME;
                fn_set_storage_data($this->key_storage, TIME + $response['expires_in']);
            }
        }

        return $response;
    }
    
    private function refreshToken($data)
    {
        $url = $this->getApiUrl() . '/oauth/token';
        
        $data_refresh = array(
            'refresh_token' => $data['refresh_token'],
            'client_id'     => $data['client_id'],
            'client_secret' => $data['client_secret'],
            'grant_type'    => 'refresh_token',
        );
        
        $response = $this->call($url, $data_refresh, false, false, false, 'refreshToken');
        
        return $response;
    }
    
    /*
    * spec_key  string
    * order_id: order_id
    * pin:      4 numbers
    * total:    total summ
    * date now
    * EX: SHA256("Номер чека(локальный счетчик на устройстве)/PIN(4 цифры)/Сумма чека(ххх.хх)/ДатаВремя(ДД.ММ.ГГГГ ЧЧ:ММ")
    */
    
    public function sendCheck($data, $order, $token)
    {
        $url = $this->getApiUrl() . '/protected/documents/registration/receipts';

        $spec_key = $this->generateSpec_key($order);
        
        $response = $this->call($url, $data, $token, true, $spec_key, 'Send check in bifit.com');

        return $response;
    }
    
    /*
    * generateSpec_key
    * return sha256 key
    */
    
    public function generateSpec_key($order)
    {
        $pin = !empty($this->addon_settings['pin']) ? $this->addon_settings['pin'] : 1111;
        
        $spec_array = array(
            $order['order_id'],
            $pin,
            sprintf("%.2f",$order['total']),
            date("d.m.Y H:i", $order['timestamp'])
        );
        $spec_key = hash('sha256', implode('/',$spec_array));
        
        return $spec_key;
    }

    /**
     * call res.
     */
    private function call($url, $data, $bearer = false, $idempotency = false, $spec_key = false, $message = '')
    {
        $extra = array(
            'headers' => array(
            'Content-Type' => 'application/x-www-form-urlencoded',
            'Cache-Control'=> 'no-cache',
            'Host' => $_SERVER['HTTP_HOST'],
            'User-Agent' => 'SwBifitOnline'
            ),
        );
        
        if($bearer !== false){
            $extra['headers']['Authorization'] = "Bearer ".$bearer['access_token'];
        }
        
        $Idempotency_key = $spec_key;
        
        if($idempotency == true){
            
            $data = json_encode($data, JSON_UNESCAPED_UNICODE);
            
            $extra['headers']['Idempotency-Key'] = $Idempotency_key;
            $extra['headers']['Content-Type'] = 'application/json';
            $extra['headers']['accept'] = '*/*';
            $extra['headers']['Content-Length'] = strlen($data);
            
            $response = self::call_curl($url, $data, $extra);
            
            $this->Logs->log($url, $data, $response, $message);
            
            if(!empty($response)){
                $response = json_decode($response, true);
            }else{
                if(strpos($response,'502 Bad Gateway')){
                    $error = 'Server Error 502';
                }
            }
            
            return $response;
            
        }else{

            try {
                $response = Http::post($url, $data, $extra);
                
                $this->Logs->log($url, json_encode($data, JSON_UNESCAPED_UNICODE), $response, $message);
    
                if(!empty($response)){
                    $response = json_decode($response, true);
                }
    
                return $response;
                
            } catch (Exception $e) {
                return $e;
            }
        }
    }
    
    
    /*
    * call_curl function
    */
   public function call_curl($url, $data, $extra, $metod = 'POST')
    { 
        $curl = curl_init();
        $header = $extra['headers'];
    
        curl_setopt_array($curl, array(
          CURLOPT_URL => $url,
          CURLOPT_RETURNTRANSFER => true,
          CURLOPT_ENCODING => "",
          CURLOPT_MAXREDIRS => 10,
          CURLOPT_TIMEOUT => 0,
          CURLOPT_FOLLOWLOCATION => true,
          CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
          CURLOPT_CUSTOMREQUEST => $metod,
          CURLOPT_POSTFIELDS =>"\r\n".$data,
          CURLOPT_HTTPHEADER => array(
            "Idempotency-Key: ".$extra['headers']['Idempotency-Key'],
            "Authorization: ".$extra['headers']['Authorization'],
            "Content-Type: ".$extra['headers']['Content-Type']
          ),
        ));
        
        $response = curl_exec($curl);
        
        curl_close($curl);
        return $response;
    }
    
    public function get_document($id, $token, $order)
    {
        if(empty($token)){
            $token = $this->getToken();
        }
        
        $url = $this->getApiUrl() . '/protected/documents/'.$id;
        
        $data = '';
        
        $Idempotency_key = $this->generateSpec_key($order);
        
        $extra['headers']['Authorization'] = "Bearer ".$token['access_token'];
        $extra['headers']['Idempotency-Key'] = $Idempotency_key;
        $extra['headers']['Content-Type'] = 'application/json';
        $extra['headers']['accept'] = '*/*';
        $extra['headers']['Content-Length'] = 0;
        
        $response = $this->call_curl($url, $data, $extra, 'GET');
        
        $message = "GET data check";
        
        if(!empty($response)){
            if(strpos($response,'error')){
                $message .= ': error'; 
            }
            
            $response = json_decode($response, true);
        }
        
        $this->Logs->log($url, $data, json_encode($response, JSON_UNESCAPED_UNICODE), $message);
        
        return $response;   
    }
    
    /**
     * Gets count receipts by conditions.
     *
     * @return int
     */    
    
    public function count(array $conditions = array())
    {
        $sql_condition = '1 = 1';

        if ($conditions) {
            $sql_condition = db_quote('?w', $conditions);
        }

        $cnt = db_get_field(
            'SELECT COUNT(*) FROM ?:bifit_online_receipts WHERE ?p',
            $sql_condition
        );

        return $cnt;
    }
    
    /**
     * Search receipts.
     *
     * @return array[]
     */
    public function search(array $conditions = array(), $limit = 20, $offset = 0)
    {
        $result = array();
        $sql_condition = '1 = 1';

        if ($conditions) {
            $sql_condition = db_quote('?w', $conditions);
        }

        $result = db_get_array(
            'SELECT * FROM ?:bifit_online_receipts WHERE ?p ORDER BY id DESC LIMIT ?i OFFSET ?i',
            $sql_condition, $limit, $offset
        );

        return $result;
    }
    
}
